@@ -36,6 +36,7 @@
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
 #include "gegl-sampler-lanczos.h"
+#include "gegl-sampler-yafr.h"
 #include "gegl-buffer-index.h"
 #include "gegl-tile-backend.h"
 #include "gegl-buffer-iterator.h"
@@ -315,15 +316,17 @@ gegl_buffer_flush (GeglBuffer *buffer)
 
 
 static void inline
-gegl_buffer_iterate (GeglBuffer *buffer,
-                     guchar     *buf,
-                     gint        rowstride,
-                     gboolean    write,
-                     const Babl *format,
-                     gint        level)
+gegl_buffer_iterate (GeglBuffer          *buffer,
+                     const GeglRectangle *roi, /* or NULL for extent */
+                     guchar              *buf,
+                     gint                 rowstride,
+                     gboolean             write,
+                     const Babl          *format,
+                     gint                 level)
 {
   gint  width       = buffer->extent.width;
   gint  height      = buffer->extent.height;
+
   gint  tile_width  = buffer->tile_storage->tile_width;
   gint  tile_height = buffer->tile_storage->tile_height;
   gint  px_size     = FMTPXS (buffer->format);
@@ -335,8 +338,10 @@ gegl_buffer_iterate (GeglBuffer *buffer,
 
   gint  buffer_shift_x = buffer->shift_x;
   gint  buffer_shift_y = buffer->shift_y;
+
   gint  buffer_x       = buffer->extent.x + buffer_shift_x;
   gint  buffer_y       = buffer->extent.y + buffer_shift_y;
+
   gint  buffer_abyss_x = buffer->abyss.x + buffer_shift_x;
   gint  buffer_abyss_y = buffer->abyss.y + buffer_shift_y;
   gint  abyss_x_total  = buffer_abyss_x + buffer->abyss.width;
@@ -344,6 +349,14 @@ gegl_buffer_iterate (GeglBuffer *buffer,
   gint  i;
   gint  factor = 1;
 
+  /* roi specified, override buffers extent */
+  if (roi)
+    {
+      width  = roi->width;
+      height = roi->height;
+      buffer_x = roi->x + buffer_shift_x;
+      buffer_y = roi->y + buffer_shift_y;
+    }
 
   for (i = 0; i < level; i++)
     {
@@ -578,8 +591,6 @@ gegl_buffer_set (GeglBuffer          *buffer,
                  void                *src,
                  gint                 rowstride)
 {
-  GeglBuffer *sub_buf;
-
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
 
 #if ENABLE_MP
@@ -594,19 +605,9 @@ gegl_buffer_set (GeglBuffer          *buffer,
     {
       pset (buffer, rect->x, rect->y, format, src);
     }
-  /* FIXME: if rect->width == TILE_WIDTH and rect->height == TILE_HEIGHT and
-   * aligned with tile grid, do a fast path, also provide helper functions
-   * for figuring out how to align accesses with the tile grid.
-   */
-  else if (rect == NULL)
-    {
-      gegl_buffer_iterate (buffer, src, rowstride, TRUE, format, 0);
-    }
   else
     {
-      sub_buf = gegl_buffer_create_sub_buffer (buffer, rect);
-      gegl_buffer_iterate (sub_buf, src, rowstride, TRUE, format, 0);
-      g_object_unref (sub_buf);
+      gegl_buffer_iterate (buffer, rect, src, rowstride, TRUE, format, 0);
     }
 
   if (gegl_buffer_is_shared(buffer))
@@ -619,26 +620,7 @@ gegl_buffer_set (GeglBuffer          *buffer,
 #endif
 }
 
-/*
- * buffer: the buffer to get data from
- * rect:   the (full size rectangle to sample)
- * dst:    the destination buffer to write to
- * format: the format to write in
- * level:  halving levels 0 = 1:1 1=1:2 2=1:4 3=1:8 ..
- *
- */
-static void
-gegl_buffer_get_scaled (GeglBuffer          *buffer,
-                        const GeglRectangle *rect,
-                        void                *dst,
-                        gint                 rowstride,
-                        const void          *format,
-                        gint                 level)
-{
-  GeglBuffer *sub_buf = gegl_buffer_create_sub_buffer (buffer, rect);
-  gegl_buffer_iterate (sub_buf, dst, rowstride, FALSE, format, level);
-  g_object_unref (sub_buf);
-}
+
 
 #if 0
 
@@ -777,19 +759,33 @@ box_filter (guint          left_weight,
   /* NOTE: this box filter presumes pre-multiplied alpha, if there
    * is alpha.
    */
-  gint i;
-  for (i = 0; i < components; i++)
+   guint lt, lm, lb;
+   guint ct, cm, cb;
+   guint rt, rm, rb;
+
+   lt = left_weight * top_weight;
+   lm = left_weight * middle_weight;
+   lb = left_weight * bottom_weight;
+   ct = center_weight * top_weight;
+   cm = center_weight * middle_weight;
+   cb = center_weight * bottom_weight;
+   rt = right_weight * top_weight;
+   rm = right_weight * middle_weight;
+   rb = right_weight * bottom_weight;
+
+#define docomponent(i) \
+      dest[i] = (src[0][i] * lt + src[3][i] * lm + src[6][i] * lb + \
+                 src[1][i] * ct + src[4][i] * cm + src[7][i] * cb + \
+                 src[2][i] * rt + src[5][i] * rm + src[8][i] * rb) / sum
+  switch (components)
     {
-      dest[i] = ( left_weight   * ((src[0][i] * top_weight) +
-                                   (src[3][i] * middle_weight) +
-                                   (src[6][i] * bottom_weight))
-                + center_weight * ((src[1][i] * top_weight) +
-                                   (src[4][i] * middle_weight) +
-                                   (src[7][i] * bottom_weight))
-                + right_weight  * ((src[2][i] * top_weight) +
-                                   (src[5][i] * middle_weight) +
-                                   (src[8][i] * bottom_weight))) / sum;
+      case 5: docomponent(4);
+      case 4: docomponent(3);
+      case 3: docomponent(2);
+      case 2: docomponent(1);
+      case 1: docomponent(0);
     }
+#undef docomponent
 }
 
 static void
@@ -970,7 +966,7 @@ gegl_buffer_get (GeglBuffer          *buffer,
 
   if (!rect && scale == 1.0)
     {
-      gegl_buffer_iterate (buffer, dest_buf, rowstride, FALSE, format, 0);
+      gegl_buffer_iterate (buffer, NULL, dest_buf, rowstride, FALSE, format, 0);
 #if ENABLE_MP
       g_static_rec_mutex_unlock (&mutex);
 #endif
@@ -986,7 +982,7 @@ gegl_buffer_get (GeglBuffer          *buffer,
     }
   if (GEGL_FLOAT_EQUAL (scale, 1.0))
     {
-      gegl_buffer_get_scaled (buffer, rect, dest_buf, rowstride, format, 0);
+      gegl_buffer_iterate (buffer, rect, dest_buf, rowstride, FALSE, format, 0);
 #if ENABLE_MP
       g_static_rec_mutex_unlock (&mutex);
 #endif
@@ -1023,13 +1019,15 @@ gegl_buffer_get (GeglBuffer          *buffer,
       buf_width          += 2;
       buf_height         += 2;
 
-
       offset_x = rect->x-floor(rect->x/scale) * scale;
       offset_y = rect->y-floor(rect->y/scale) * scale;
 
-
       sample_buf = g_malloc (buf_width * buf_height * bpp);
-      gegl_buffer_get_scaled (buffer, &sample_rect, sample_buf, GEGL_AUTO_ROWSTRIDE, format, level);
+      gegl_buffer_iterate (buffer, &sample_rect, sample_buf, GEGL_AUTO_ROWSTRIDE, FALSE, format, level);
+#if 1
+  /* slows testing of rendering code speed to much for now and
+   * no time to make a fast implementation
+   */
 
       if (BABL (format)->format.type[0] == (BablType *) babl_type ("u8")
           && !(level == 0 && scale > 1.99))
@@ -1051,6 +1049,7 @@ gegl_buffer_get (GeglBuffer          *buffer,
                                  rowstride);
         }
       else
+#endif
         {
           resample_nearest (dest_buf,
                             sample_buf,
@@ -1203,12 +1202,13 @@ gegl_buffer_clear (GeglBuffer          *dst,
 
   pxsize = dst->format->format.bytes_per_pixel;
 
+  /* FIXME: this can be even further optimized by special casing it so
+   * that fully voided tiles are dropped.
+   */
   i = gegl_buffer_iterator_new (dst, dst_rect, dst->format, GEGL_BUFFER_WRITE);
   while (gegl_buffer_iterator_next (i))
     {
-      gint j;
-      for (j=0;j<i->length * pxsize;j++)
-        ((guchar*)(i->data[0]))[j]=0;
+      memset (((guchar*)(i->data[0])), 0, i->length * pxsize);
     }
 }
 
