@@ -67,6 +67,8 @@ struct _GeglNodePrivate
   GeglNode       *parent;
   gchar          *name;
   GeglProcessor  *processor;
+  GeglEvalMgr    *eval_mgr;
+  GHashTable     *contexts;
 };
 
 
@@ -149,7 +151,7 @@ gegl_node_class_init (GeglNodeClass *klass)
   g_object_class_install_property (gobject_class, PROP_DONT_CACHE,
                                    g_param_spec_boolean ("dont-cache",
                                                          "Do not cache",
-                                                        "Do not cache the result of this operation, the property is inherithed by children created from a node.",
+                                                        "Do not cache the result of this operation, the property is inherited by children created from a node.",
                                                         TRUE,
                                                         G_PARAM_READWRITE));
 
@@ -208,6 +210,7 @@ gegl_node_init (GeglNode *self)
   priv->children    = NULL;
   priv->name        = NULL;
   priv->processor   = NULL;
+  priv->contexts    = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -241,6 +244,12 @@ dispose (GObject *gobject)
       self->cache = NULL;
     }
 
+  if (priv->eval_mgr)
+    {
+      g_object_unref (priv->eval_mgr);
+      priv->eval_mgr = NULL;
+    }
+
   if (priv->processor)
     {
       gegl_processor_destroy (priv->processor);
@@ -278,6 +287,7 @@ finalize (GObject *gobject)
     {
       g_free (priv->name);
     }
+  g_hash_table_destroy (priv->contexts);
 
   G_OBJECT_CLASS (gegl_node_parent_class)->finalize (gobject);
 }
@@ -306,6 +316,9 @@ set_property (GObject      *gobject,
                          * the preceding argument is NULL, gcc might warn about
                          * use of uninitialized variable.
                          */
+#if defined(__GNUC__)
+          memset(&null, 0, sizeof(null));
+#endif
           gegl_node_set_op_class (node, g_value_get_string (value), NULL, null);
         }
         break;
@@ -523,15 +536,19 @@ gegl_node_connect_to (GeglNode    *source,
 
 void
 gegl_node_invalidated (GeglNode            *node,
-                       const GeglRectangle *rect)
+                       const GeglRectangle *rect,
+                       gboolean             clear_cache)
 {
   g_return_if_fail (GEGL_IS_NODE (node));
   g_return_if_fail (rect != NULL);
 
   if (node->cache)
     {
+      if (rect && clear_cache)
+        gegl_buffer_clear (GEGL_BUFFER (node->cache), rect);
       gegl_cache_invalidate (node->cache, rect);
     }
+  node->valid_have_rect = FALSE;
 
   g_signal_emit (node, gegl_node_signals[INVALIDATED], 0,
                  rect, NULL);
@@ -565,7 +582,7 @@ source_invalidated (GeglNode            *source,
       dirty_rect = *rect;
     }
 
-  gegl_node_invalidated (destination, &dirty_rect);
+  gegl_node_invalidated (destination, &dirty_rect, FALSE);
 }
 
 gboolean
@@ -777,20 +794,35 @@ gegl_node_link_many (GeglNode *source,
   va_end (var_args);
 }
 
+static void ensure_eval_mgr (GeglNode *node,
+                             const gchar *pad)
+{
+  GeglNodePrivate *priv;
+  priv = GEGL_NODE_GET_PRIVATE (node);
+  if (!priv->eval_mgr)
+    priv->eval_mgr = gegl_eval_mgr_new (node, pad);
+}
+
 static GeglBuffer *
 gegl_node_apply_roi (GeglNode            *self,
                      const gchar         *output_pad_name,
                      const GeglRectangle *roi)
 {
-  GeglEvalMgr *eval_mgr;
+  GeglNodePrivate *priv;
   GeglBuffer  *buffer;
 
-  eval_mgr      = g_object_new (GEGL_TYPE_EVAL_MGR, NULL);
-  eval_mgr->roi = *roi;
-  buffer        = gegl_eval_mgr_apply (eval_mgr, self, output_pad_name);
-
-  g_object_unref (eval_mgr);
+  priv = GEGL_NODE_GET_PRIVATE (self);
+  ensure_eval_mgr (self, output_pad_name);
 
+  if (roi)
+    {
+      priv->eval_mgr->roi = *roi;
+    }
+  else
+    {
+      priv->eval_mgr->roi = gegl_node_get_bounding_box (self);
+    }
+  buffer = gegl_eval_mgr_apply (priv->eval_mgr);
   return buffer;
 }
 
@@ -806,9 +838,20 @@ gegl_node_blit (GeglNode            *node,
   g_return_if_fail (GEGL_IS_NODE (node));
   g_return_if_fail (roi != NULL);
 
+#if 0
+  if (flags == GEGL_BLIT_DEFAULT)
+    flags = GEGL_BLIT_CACHE;
+#endif
+
+  /* temporarily made blit use caching, but render
+   * blocking, this to be able to have less coupling
+   * with the processor
+   */
+#if 1
   if (flags == GEGL_BLIT_DEFAULT)
     {
       GeglBuffer *buffer;
+
       buffer = gegl_node_apply_roi (node, "output", roi);
       if (buffer && destination_buf)
         {
@@ -827,16 +870,20 @@ gegl_node_blit (GeglNode            *node,
       if (buffer)
         g_object_unref (buffer);
     }
-  else if ((flags & GEGL_BLIT_CACHE) ||
+  else 
+#endif
+    
+    if ((flags & GEGL_BLIT_CACHE) ||
            (flags & GEGL_BLIT_DIRTY))
     {
       GeglCache *cache = gegl_node_get_cache (node);
       if (!(flags & GEGL_BLIT_DIRTY))
-        { /* if we're not blitting dirtily, we need to make sure
-             that the data is available */
-          GeglProcessor *processor = gegl_node_new_processor (node, roi);
-          while (gegl_processor_work (processor, NULL)) ;
-          g_object_unref (G_OBJECT (processor));
+        { 
+          GeglNodePrivate *priv = GEGL_NODE_GET_PRIVATE (node);
+          if (!priv->processor) 
+           priv->processor = gegl_node_new_processor (node, roi);
+          gegl_processor_set_rectangle (priv->processor, roi);
+          while (gegl_processor_work (priv->processor, NULL)) ;
         }
       if (destination_buf)
         {
@@ -846,17 +893,6 @@ gegl_node_blit (GeglNode            *node,
     }
 }
 
-GeglBuffer *
-gegl_node_apply (GeglNode    *self,
-                 const gchar *output_prop_name)
-{
-  GeglRectangle defined;
-
-  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
-
-  defined = gegl_node_get_bounding_box (self);
-  return gegl_node_apply_roi (self, "output", &defined);
-}
 
 GSList *
 gegl_node_get_depends_on (GeglNode *self)
@@ -939,13 +975,13 @@ gegl_node_set_op_class (GeglNode    *node,
       if (!type)
         {
           g_warning ("Failed to set operation type %s, using a passthrough op instead", op_class);
-          if (strcmp (op_class, "nop"))
+          if (strcmp (op_class, "gegl:nop"))
             {
-              gegl_node_set_op_class (node, "nop", NULL, var_args);
+              gegl_node_set_op_class (node, "gegl:nop", NULL, var_args);
             }
           else
             {
-              g_warning ("The failing op was 'nop' this means that GEGL was unable to locate any of it's\n"
+              g_warning ("The failing op was 'gegl:nop' this means that GEGL was unable to locate any of it's\n"
                          "plug-ins. Try making GEGL_PATH point to the directory containing the .so|.dll\n"
                          "files with the image processing plug-ins, optionally you could try to make it\n"
                          "point to the operations directory of a GEGL sourcetree with a build.");
@@ -968,6 +1004,15 @@ gegl_node_set_op_class (GeglNode    *node,
     }
 }
 
+static gboolean
+invalidate_have_rect (GObject    *gobject,
+                      gpointer    foo,
+                      gpointer    user_data)
+{
+  GEGL_NODE (user_data)->valid_have_rect = FALSE;
+  return TRUE;
+}
+
 
 static void
 property_changed (GObject    *gobject,
@@ -1006,7 +1051,7 @@ property_changed (GObject    *gobject,
                                        &dirty_rect,
                                        &new_have_rect);*/
 
-          gegl_node_invalidated (self, &dirty_rect);
+          gegl_node_invalidated (self, &dirty_rect, FALSE);
         }
       else
         {
@@ -1021,7 +1066,7 @@ property_changed (GObject    *gobject,
                                        &dirty_rect,
                                        &new_have_rect);
 
-          gegl_node_invalidated (self, &dirty_rect);
+          gegl_node_invalidated (self, &dirty_rect, FALSE);
         }
     }
 }
@@ -1093,6 +1138,7 @@ gegl_node_set_operation_object (GeglNode      *self,
       g_free (output_dest_pad);
   }
 
+  g_signal_connect (G_OBJECT (operation), "notify", G_CALLBACK (invalidate_have_rect), self);
   g_signal_connect (G_OBJECT (operation), "notify", G_CALLBACK (property_changed), self);
   property_changed (G_OBJECT (operation), (GParamSpec *) self, self);
 }
@@ -1394,22 +1440,15 @@ gegl_node_get_operation (const GeglNode *node)
 void
 gegl_node_set_need_rect (GeglNode *node,
                          gpointer  context_id,
-                         gint      x,
-                         gint      y,
-                         gint      width,
-                         gint      height)
+                         const GeglRectangle *rect)
 {
-  GeglNodeContext *context;
+  GeglOperationContext *context;
 
   g_return_if_fail (GEGL_IS_NODE (node));
   g_return_if_fail (context_id != NULL);
 
   context = gegl_node_get_context (node, context_id);
-
-  context->need_rect.x      = x;
-  context->need_rect.y      = y;
-  context->need_rect.width  = width;
-  context->need_rect.height = height;
+  gegl_operation_context_set_need_rect (context, rect);
 }
 
 const gchar *
@@ -1482,6 +1521,10 @@ gegl_node_get_bounding_box (GeglNode *root)
 
   if (!root)
     return dummy;
+
+  if (root->valid_have_rect)
+    return root->have_rect;
+
   pad = gegl_node_get_pad (root, "output");
   if (pad && pad->node != root)
     {
@@ -1511,12 +1554,16 @@ gegl_node_get_bounding_box (GeglNode *root)
   g_object_unref (root);
   g_free (id);
 
+  root->valid_have_rect = TRUE;
   return root->have_rect;
 }
 
+#if 1
+
 void
 gegl_node_process (GeglNode *self)
 {
+  /* XXX: should perhaps use the internal processor? */
   GeglProcessor *processor;
 
   g_return_if_fail (GEGL_IS_NODE (self));
@@ -1527,7 +1574,7 @@ gegl_node_process (GeglNode *self)
   gegl_processor_destroy (processor);
 }
 
-#if 0
+#else
 /* simplest form of GeglProcess that processes all data in one
  *
  * single large chunk
@@ -1536,7 +1583,7 @@ void
 gegl_node_process (GeglNode *self)
 {
   GeglNode        *input;
-  GeglNodeContext *context;
+  GeglOperationContext *context;
   GeglBuffer      *buffer;
   GeglRectangle    defined;
 
@@ -1555,44 +1602,33 @@ gegl_node_process (GeglNode *self)
     GValue value = { 0, };
     g_value_init (&value, GEGL_TYPE_BUFFER);
     g_value_set_object (&value, buffer);
-    gegl_node_context_set_property (context, "input", &value);
+    gegl_operation_context_set_property (context, "input", &value);
     g_value_unset (&value);
   }
 
-  gegl_node_context_set_result_rect (context, defined.x, defined.y, defined.width, defined.h);
-  gegl_operation_process (self->operation, &defined, "foo");
+  gegl_operation_context_set_result_rect (context, &defined);
+  gegl_operation_process (self->operation, context, "output", &defined);
   gegl_node_remove_context (self, &defined);
   g_object_unref (buffer);
 }
 #endif
 
-static gint
-lookup_context (gconstpointer a,
-                gconstpointer context_id)
-{
-  GeglNodeContext *context = (void *) a;
-
-  if (context->context_id == context_id)
-    return 0;
-  return -1;
-}
-
 void babl_backtrack (void);
 
-GeglNodeContext *
+GeglOperationContext *
 gegl_node_get_context (GeglNode *self,
                        gpointer  context_id)
 {
-  GSList          *found;
-  GeglNodeContext *context = NULL;
+  GeglOperationContext *context = NULL;
+  GeglNodePrivate *priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
   g_return_val_if_fail (context_id != NULL, NULL);
 
-  found = g_slist_find_custom (self->context, context_id, lookup_context);
-  if (found)
-    context = found->data;
-  else
+  priv = GEGL_NODE_GET_PRIVATE (self);
+
+  context = g_hash_table_lookup (priv->contexts, context_id);
+  if (!context)
     {
       g_warning ("didn't find %p", context_id);
       babl_backtrack ();
@@ -1604,7 +1640,8 @@ void
 gegl_node_remove_context (GeglNode *self,
                           gpointer  context_id)
 {
-  GeglNodeContext *context;
+  GeglOperationContext *context;
+  GeglNodePrivate *priv;
 
   g_return_if_fail (GEGL_IS_NODE (self));
   g_return_if_fail (context_id != NULL);
@@ -1616,24 +1653,23 @@ gegl_node_remove_context (GeglNode *self,
                  context_id, gegl_node_get_debug_name (self));
       return;
     }
-  self->context = g_slist_remove (self->context, context);
-  g_object_unref (context);
+  priv = GEGL_NODE_GET_PRIVATE (self);
+  g_hash_table_remove (priv->contexts, context_id);
+  gegl_operation_context_destroy (context);
 }
 
-GeglNodeContext *
+GeglOperationContext *
 gegl_node_add_context (GeglNode *self,
                        gpointer  context_id)
 {
-  GeglNodeContext *context = NULL;
-  GSList          *found;
+  GeglOperationContext *context = NULL;
+  GeglNodePrivate *priv;
 
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
   g_return_val_if_fail (context_id != NULL, NULL);
 
-  found = g_slist_find_custom (self->context, context_id, lookup_context);
-
-  if (found)
-    context = found->data;
+  priv = GEGL_NODE_GET_PRIVATE (self);
+  context = g_hash_table_lookup (priv->contexts, context_id);
 
   if (context)
     {
@@ -1642,10 +1678,9 @@ gegl_node_add_context (GeglNode *self,
       return context;
     }
 
-  context             = g_object_new (GEGL_TYPE_NODE_CONTEXT, NULL);
-  context->node       = self;
-  context->context_id = context_id;
-  self->context       = g_slist_prepend (self->context, context);
+  context             = gegl_operation_context_new ();
+  context->operation  = self->operation;
+  g_hash_table_insert (priv->contexts, context_id, context);
   return context;
 }
 
@@ -1696,7 +1731,7 @@ gegl_node_insert_before (GeglNode *self,
   gegl_node_link_many (other, to_be_inserted, self, NULL);
 
   /* emit the change ourselves */
-  gegl_node_invalidated (self, &rectangle);
+  gegl_node_invalidated (self, &rectangle, FALSE);
 }
 
 gint
@@ -2062,7 +2097,7 @@ gegl_node_get_pad_proxy (GeglNode    *graph,
   pad = gegl_node_get_pad (node, name);
   if (!pad)
     {
-      GeglNode *nop     = g_object_new (GEGL_TYPE_NODE, "operation", "nop", "name", is_graph_input ? "proxynop-input" : "proxynop-output", NULL);
+      GeglNode *nop     = g_object_new (GEGL_TYPE_NODE, "operation", "gegl:nop", "name", is_graph_input ? "proxynop-input" : "proxynop-output", NULL);
       GeglPad  *nop_pad = gegl_node_get_pad (nop, is_graph_input ? "input" : "output");
       gegl_node_add_child (graph, nop);
       g_object_unref (nop); /* our reference is made by the
