@@ -17,38 +17,47 @@
  */
 
 #include "config.h"
+
 #include <glib-object.h>
-#include "gegl-types.h"
-#include "graph/gegl-node.h"
-#include "gegl-processor.h"
-#include "gegl-utils.h"
+
 #include "buffer/gegl-region.h"
+
+#include "graph/gegl-node.h"
+
 #include "operation/gegl-operation-sink.h"
+
 #include "gegl-config.h"
+#include "gegl-processor.h"
+#include "gegl-types.h"
+#include "gegl-utils.h"
+
 
 enum
 {
   PROP_0,
   PROP_NODE,
   PROP_CHUNK_SIZE,
-  PROP_PROGRESS
+  PROP_PROGRESS,
+  PROP_RECTANGLE
 };
 
-static void      gegl_processor_class_init (GeglProcessorClass    *klass);
-static void      gegl_processor_init       (GeglProcessor         *self);
-static void      finalize                  (GObject               *self_object);
-static void      set_property              (GObject               *gobject,
-                                            guint                  prop_id,
-                                            const GValue          *value,
-                                            GParamSpec            *pspec);
-static void      get_property              (GObject               *gobject,
-                                            guint                  prop_id,
-                                            GValue                *value,
-                                            GParamSpec            *pspec);
-static GObject * constructor               (GType                  type,
-                                            guint                  n_params,
-                                            GObjectConstructParam *params);
-static gdouble   gegl_processor_progress   (GeglProcessor         *processor);
+
+static void      gegl_processor_class_init   (GeglProcessorClass    *klass);
+static void      gegl_processor_init         (GeglProcessor         *self);
+static void      gegl_processor_finalize     (GObject               *self_object);
+static void      gegl_processor_set_property (GObject               *gobject,
+                                              guint                  prop_id,
+                                              const GValue          *value,
+                                              GParamSpec            *pspec);
+static void      gegl_processor_get_property (GObject               *gobject,
+                                              guint                  prop_id,
+                                              GValue                *value,
+                                              GParamSpec            *pspec);
+static GObject * gegl_processor_constructor  (GType                  type,
+                                              guint                  n_params,
+                                              GObjectConstructParam *params);
+static gdouble   gegl_processor_progress     (GeglProcessor         *processor);
+
 
 struct _GeglProcessor
 {
@@ -56,46 +65,58 @@ struct _GeglProcessor
   GeglNode        *node;
   GeglRectangle    rectangle;
   GeglNode        *input;
-  GeglNodeContext *context;
+  GeglOperationContext *context;
 
-  GeglRegion *valid_region; /* used when doing unbuffered rendering */
-  GeglRegion *queued_region;
-  GSList     *dirty_rectangles;
-  gint        chunk_size;
+  GeglRegion      *valid_region;     /* used when doing unbuffered rendering */
+  GeglRegion      *queued_region;
+  GSList          *dirty_rectangles;
+  gint             chunk_size;
 
-  GThread *thread;
-  gboolean thread_done;
-  gdouble  progress;
+  GThread         *thread;
+  gboolean         thread_done;
+  gdouble          progress;
 };
 
+
 G_DEFINE_TYPE (GeglProcessor, gegl_processor, G_TYPE_OBJECT);
 
+
 static void
 gegl_processor_class_init (GeglProcessorClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  gobject_class->finalize     = finalize;
-  gobject_class->constructor  = constructor;
-  gobject_class->set_property = set_property;
-  gobject_class->get_property = get_property;
+  gobject_class->finalize     = gegl_processor_finalize;
+  gobject_class->constructor  = gegl_processor_constructor;
+  gobject_class->set_property = gegl_processor_set_property;
+  gobject_class->get_property = gegl_processor_get_property;
 
   g_object_class_install_property (gobject_class, PROP_NODE,
                                    g_param_spec_object ("node",
                                                         "GeglNode",
-                                                        "The GeglNode to process (will saturate the providers cache if it the provided node is a sink node)",
+                                                        "The GeglNode to process (will saturate the provider's cache if the provided node is a sink node)",
                                                         GEGL_TYPE_NODE,
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT));
 
+  g_object_class_install_property (gobject_class, PROP_RECTANGLE,
+                                   g_param_spec_pointer ("rectangle",
+                                                         "rectangle",
+                                                         "The rectangle of the region to process.",
+                                                         G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_PROGRESS,
-                                   g_param_spec_double ("progress", "progress", "query progress 0.0 is not started 1.0 is done.",
-                                                     0.0, 1.0, 0.0,
-                                                     G_PARAM_READWRITE));
+                                   g_param_spec_double ("progress",
+                                                        "progress",
+                                                        "query progress; 0.0 is not started, 1.0 is done.",
+                                                        0.0, 1.0, 0.0,
+                                                        G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_CHUNK_SIZE,
-                                   g_param_spec_int ("chunksize", "chunksize", "Size of chunks being rendered (larger chunks need more memory to do the processing).",
-                                                     1, 1024*1024, 
-                                                     gegl_config()->chunk_size, 
+                                   g_param_spec_int ("chunksize",
+                                                     "chunksize",
+                                                     "Size of chunks being rendered (larger chunks need more memory to do the processing).",
+                                                     1, 1024 * 1024, gegl_config()->chunk_size, 
                                                      G_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT_ONLY));
 }
@@ -111,11 +132,10 @@ gegl_processor_init (GeglProcessor *processor)
   processor->chunk_size       = 128 * 128;
 }
 
-
 static GObject *
-constructor (GType                  type,
-             guint                  n_params,
-             GObjectConstructParam *params)
+gegl_processor_constructor (GType                  type,
+                            guint                  n_params,
+                            GObjectConstructParam *params)
 {
   GObject       *object;
   GeglProcessor *processor;
@@ -129,15 +149,20 @@ constructor (GType                  type,
     {
       processor->input = gegl_node_get_producer (processor->node, "input", NULL);
       if (!gegl_operation_sink_needs_full (processor->node->operation))
-        processor->valid_region = gegl_region_new ();
+        {
+          processor->valid_region = gegl_region_new ();
+        }
       else
-        processor->valid_region = NULL;
+        {
+          processor->valid_region = NULL;
+        }
     }
   else
     {
       processor->input = processor->node;
       processor->valid_region = NULL;
     }
+
   g_object_ref (processor->input);
 
   processor->queued_region = gegl_region_new ();
@@ -145,30 +170,39 @@ constructor (GType                  type,
   return object;
 }
 
-
 static void
-finalize (GObject *self_object)
+gegl_processor_finalize (GObject *self_object)
 {
   GeglProcessor *processor = GEGL_PROCESSOR (self_object);
 
   if (processor->node)
-    g_object_unref (processor->node);
+    {
+      g_object_unref (processor->node);
+    }
+
   if (processor->input)
-    g_object_unref (processor->input);
+    {
+      g_object_unref (processor->input);
+    }
+
   if (processor->queued_region)
-    gegl_region_destroy (processor->queued_region);
+    {
+      gegl_region_destroy (processor->queued_region);
+    }
+
   if (processor->valid_region)
-    gegl_region_destroy (processor->valid_region);
+    {
+      gegl_region_destroy (processor->valid_region);
+    }
 
   G_OBJECT_CLASS (gegl_processor_parent_class)->finalize (self_object);
 }
 
-
 static void
-set_property (GObject      *gobject,
-              guint         property_id,
-              const GValue *value,
-              GParamSpec   *pspec)
+gegl_processor_set_property (GObject      *gobject,
+                             guint         property_id,
+                             const GValue *value,
+                             GParamSpec   *pspec)
 {
   GeglProcessor *self = GEGL_PROCESSOR (gobject);
 
@@ -186,6 +220,10 @@ set_property (GObject      *gobject,
         self->chunk_size = g_value_get_int (value);
         break;
 
+      case PROP_RECTANGLE:
+        gegl_processor_set_rectangle (self, g_value_get_pointer (value));
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
         break;
@@ -193,10 +231,10 @@ set_property (GObject      *gobject,
 }
 
 static void
-get_property (GObject    *gobject,
-              guint       property_id,
-              GValue     *value,
-              GParamSpec *pspec)
+gegl_processor_get_property (GObject    *gobject,
+                             guint       property_id,
+                             GValue     *value,
+                             GParamSpec *pspec)
 {
   GeglProcessor *self = GEGL_PROCESSOR (gobject);
 
@@ -206,6 +244,10 @@ get_property (GObject    *gobject,
         g_value_set_object (value, self->node);
         break;
 
+      case PROP_RECTANGLE:
+        g_value_set_pointer (value, &self->rectangle);
+        break;
+
       case PROP_CHUNK_SIZE:
         g_value_set_int (value, self->chunk_size);
         break;
@@ -224,20 +266,37 @@ gegl_processor_set_rectangle (GeglProcessor       *processor,
                               const GeglRectangle *rectangle)
 {
   GSList        *iter;
-  GeglRectangle  bounds;
+  GeglRectangle  input_bounding_box;
 
-  if (gegl_rectangle_equal (&processor->rectangle, rectangle))
-    return;
+  if (! rectangle)
+    {
+      input_bounding_box = gegl_node_get_bounding_box (processor->input);
+      rectangle          = &input_bounding_box;
+    }
 
-  bounds               = gegl_node_get_bounding_box (processor->input);
-  processor->rectangle = *rectangle;
-  gegl_rectangle_intersect (&processor->rectangle, &processor->rectangle, &bounds);
+  if (! gegl_rectangle_equal (&processor->rectangle, rectangle))
+    {
 
-  /* remove already queued dirty rectangles */
-  for (iter = processor->dirty_rectangles; iter; iter = g_slist_next (iter))
-    g_slice_free (GeglRectangle, iter->data);
-  g_slist_free (processor->dirty_rectangles);
-  processor->dirty_rectangles = NULL;
+#if 0
+    /* XXX: this is a large penalty hit, so we assume the rectangle
+     * we're getting is part of the bounding box we're hitting */
+      GeglRectangle  bounds;
+      bounds               = processor->bounds;/*gegl_node_get_bounding_box (processor->input);*/
+#endif
+      processor->rectangle = *rectangle;
+#if 0
+      gegl_rectangle_intersect (&processor->rectangle, &processor->rectangle, &bounds);
+#endif
+
+      /* remove already queued dirty rectangles */
+      for (iter = processor->dirty_rectangles; iter; iter = g_slist_next (iter))
+        {
+          g_slice_free (GeglRectangle, iter->data);
+        }
+      g_slist_free (processor->dirty_rectangles);
+
+      processor->dirty_rectangles = NULL;
+    }
 }
 
 GeglProcessor *
@@ -248,25 +307,20 @@ gegl_node_new_processor (GeglNode            *node,
 
   g_return_val_if_fail (GEGL_IS_NODE (node), NULL);
 
-  processor = g_object_new (GEGL_TYPE_PROCESSOR, "node", node, NULL);
+  processor = g_object_new (GEGL_TYPE_PROCESSOR,
+                            "node",      node,
+                            "rectangle", rectangle,
+                            NULL);
 
-  if (rectangle)
+  /* FIXME: Look for what pads that are available rather than looking
+   * at what type of operation we are dealing with
+   */
+  if (node->operation                          &&
+      GEGL_IS_OPERATION_SINK (node->operation) &&
+      gegl_operation_sink_needs_full (node->operation))
     {
-      gegl_processor_set_rectangle (processor, rectangle);
-    }
-  else
-    {
-      GeglRectangle tmp = gegl_node_get_bounding_box (processor->input);
-      gegl_processor_set_rectangle (processor, &tmp);
-    }
+      GeglCache *cache;
 
-  if (node->operation &&
-      GEGL_IS_OPERATION_SINK (node->operation))
-    {
-      GeglCache     *cache;
-
-      if (!gegl_operation_sink_needs_full (node->operation))
-          return processor;
       cache = gegl_node_get_cache (processor->input);
 
       processor->context = gegl_node_add_context (node, cache);
@@ -274,26 +328,14 @@ gegl_node_new_processor (GeglNode            *node,
         GValue value = { 0, };
         g_value_init (&value, GEGL_TYPE_BUFFER);
         g_value_set_object (&value, cache);
-        gegl_node_context_set_property (processor->context, "input", &value);
+        gegl_operation_context_set_property (processor->context, "input", &value);
         g_value_unset (&value);
       }
 
-      gegl_node_context_set_result_rect (processor->context,
-                                         processor->rectangle.x,
-                                         processor->rectangle.y,
-                                         processor->rectangle.width,
-                                         processor->rectangle.height);
-      gegl_node_context_set_need_rect   (processor->context,
-                                         processor->rectangle.x,
-                                         processor->rectangle.y,
-                                         processor->rectangle.width,
-                                         processor->rectangle.height);
-
-
-    }
-  else
-    {
-      processor->context = NULL;
+      gegl_operation_context_set_result_rect (processor->context,
+                                              &processor->rectangle);
+      gegl_operation_context_set_need_rect   (processor->context,
+                                              &processor->rectangle);
     }
 
   return processor;
@@ -324,29 +366,26 @@ render_rectangle (GeglProcessor *processor)
         {
           gint band_size;
 
-          if (dr->height > dr->width)
-            {
-              GeglRectangle *fragment;
-
-              band_size = dr->height / 2;
-
-              if (band_size < 1)
-                band_size = 1;
-
-              fragment = g_slice_dup (GeglRectangle, dr);
-
-              fragment->height = band_size;
-              dr->height      -= band_size;
-              dr->y           += band_size;
-
-              processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, fragment);
-              return TRUE;
-            }
-          else
+          if (dr->width > dr->height)
             {
               GeglRectangle *fragment;
 
+/* try to make the rects generated match better with potential 2^n sized
+ * tiles, XXX: should be improved to make the next slice fit as well.
+ */
               band_size = dr->width / 2;
+#if 1
+              if (band_size <= 256)
+                {
+                  band_size = MIN(band_size, 128); /* prefer a band_size of 128,
+                                                      hoping to hit tiles */
+                }
+              else if (band_size <= 512)
+                {
+                  band_size = MIN(band_size, 256); /* prefer a band_size of 128,
+                                                      hoping to hit tiles */
+                }
+#endif
 
               if (band_size < 1)
                 band_size = 1;
@@ -358,6 +397,38 @@ render_rectangle (GeglProcessor *processor)
               dr->x          += band_size;
 
               processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, fragment);
+
+              return TRUE;
+            }
+          else
+            {
+              GeglRectangle *fragment;
+
+              band_size = dr->height / 2;
+
+
+              if (band_size <= 256)
+                {
+                  band_size = MIN(band_size, 128); /* prefer a band_size of 128,
+                                                      hoping to hit tiles */
+                }
+              else if (band_size <= 512)
+                {
+                  band_size = MIN(band_size, 256); /* prefer a band_size of 128,
+                                                      hoping to hit tiles */
+                }
+
+              if (band_size < 1)
+                band_size = 1;
+
+              fragment = g_slice_dup (GeglRectangle, dr);
+
+              fragment->height = band_size;
+              dr->height      -= band_size;
+              dr->y           += band_size;
+
+              processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles, fragment);
+
               return TRUE;
             }
         }
@@ -366,9 +437,9 @@ render_rectangle (GeglProcessor *processor)
       if (!dr->width || !dr->height)
         {
           g_slice_free (GeglRectangle, dr);
+
           return TRUE;
         }
-
 	
       if (buffered)
         {
@@ -437,7 +508,7 @@ region_area (GeglRegion *region)
 
 static gint
 area_left (GeglRegion    *area,
-                       GeglRectangle *rectangle)
+           GeglRectangle *rectangle)
 {
   GeglRegion *region;
   gint        sum = 0;
@@ -462,14 +533,18 @@ static gdouble
 gegl_processor_progress (GeglProcessor *processor)
 {
   GeglRegion *valid_region;
-  gint valid;
-  gint wanted;
-  gdouble ret;
+  gint        valid;
+  gint        wanted;
+  gdouble     ret;
 
   if (processor->valid_region)
-    valid_region = processor->valid_region;
+    {
+      valid_region = processor->valid_region;
+    }
   else
-    valid_region = gegl_node_get_cache (processor->input)->valid_region;
+    {
+      valid_region = gegl_node_get_cache (processor->input)->valid_region;
+    }
 
   wanted = rect_area (&(processor->rectangle));
   valid  = wanted - area_left (valid_region, &(processor->rectangle));
@@ -479,12 +554,16 @@ gegl_processor_progress (GeglProcessor *processor)
         return 1.0;
       return 0.999;
     }
+
   ret = (double) valid / wanted;
   if (ret>=1.0)
     {
       if (!gegl_processor_is_rendered (processor))
-        return 0.9999;
+        {
+          return 0.9999;
+        }
     }
+
   return ret;
 }
 
@@ -496,12 +575,17 @@ gegl_processor_render (GeglProcessor *processor,
   GeglRegion *valid_region;
 
   if (processor->valid_region)
-    valid_region = processor->valid_region;
+    {
+      valid_region = processor->valid_region;
+    }
   else
-    valid_region = gegl_node_get_cache (processor->input)->valid_region;
+    {
+      valid_region = gegl_node_get_cache (processor->input)->valid_region;
+    }
 
   {
     gboolean more_work = render_rectangle (processor);
+
     if (more_work == TRUE)
       {
         if (progress)
@@ -528,6 +612,7 @@ gegl_processor_render (GeglProcessor *processor,
               }
             wanted = 1;
           }
+
         return more_work;
       }
   }
@@ -554,6 +639,7 @@ gegl_processor_render (GeglProcessor *processor,
           processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles,
                                                          g_slice_dup (GeglRectangle, &roi));
         }
+
       g_free (rectangles);
 
       if (n_rectangles != 0)
@@ -563,6 +649,7 @@ gegl_processor_render (GeglProcessor *processor,
                                rect_area (rectangle));
           return TRUE;
         }
+
       return FALSE;
     }
   else if (!gegl_region_empty (processor->queued_region) &&
@@ -587,10 +674,14 @@ gegl_processor_render (GeglProcessor *processor,
           processor->dirty_rectangles = g_slist_prepend (processor->dirty_rectangles,
                                                          g_slice_dup (GeglRectangle, &roi));
         }
+
       g_free (rectangles);
     }
+
   if (progress)
-    *progress = 0.69;
+    {
+      *progress = 0.69;
+    }
 
   return !gegl_processor_is_rendered (processor);
 }
@@ -608,6 +699,11 @@ gegl_processor_work (GeglProcessor *processor,
       return TRUE;
     }
 
+  if (progress)
+    {
+      *progress = 1.0;
+    }
+
   if (processor->context)
     {
       /* the actual writing to the destination */
@@ -618,14 +714,10 @@ gegl_processor_work (GeglProcessor *processor,
                               );
       gegl_node_remove_context (processor->node, cache);
       processor->context = NULL;
-      if (progress)
-        *progress = 1.0;
+
       return TRUE;
     }
 
-  if (progress)
-    *progress = 1.0;
-
   return FALSE;
 }
 
